# -*- coding: utf-8 -*-
"""
Created on Mon Apr  9 17:03:32 2018

@author: eivin
"""

from abaqus import *
from abaqusConstants import *
import __main__

import datetime
import section
import regionToolset
import displayGroupMdbToolset as dgm
import part
import material
import assembly
import step
import interaction
import load
import mesh
import optimization
import job
import sketch
import visualization
import xyPlot
import displayGroupOdbToolset as dgo
import connectorBehavior
import sys


def getUsefulPar(fileName):
    file = open(fileName)
    for line in file:
        par = line.split()
    modelName = par[0]
    dX = float(par[1])
    dY = float(par[2])
    th = float(par[3])
    ky = float(par[4])
    file.close()
    return(modelName,dX,dY,th,ky)
    
def results(modelName,dX,dY,th,ky):  
    # Non-linear
    odbName = modelName+'.odb'
    odb = session.openOdb(odbName)
    
    # The sets of nodes found in the result file:
    regOdbYB=odb.rootAssembly.nodeSets['SET-NODES-YB']
#    dispNode=odb.rootAssembly.nodeSets['SET-NODES-YB'].nodes[0][0]
    momentNode=odb.rootAssembly.nodeSets['SET-NODE-MOMENT']
    outFileName=odbName.split('.')[0]+'.txt'
    
    
    C=0
    curvature = ky/numSteps
    for i in range(numSteps):
        ip=str(i+1)
        res=''
        
        par=curvature/len(odb.steps['Step-Contact-'+ip].frames)
        
        if i == 0:
            f=open(outFileName,'w')
        else:
            f=open(outFileName,'a')
        for frame in odb.steps['Step-Contact-'+ip].frames:
            C=C+par
            rfsYB=frame.fieldOutputs['RF'].getSubset(region=regOdbYB).values
            momentNode=odb.rootAssembly.nodeSets['SET-NODE-MOMENT']
            y0=momentNode.nodes[0][0].coordinates[1]
            z0=momentNode.nodes[0][0].coordinates[2]
            
            sumMy=0
            for counter, rfYB in enumerate(rfsYB):
                
                rfy=rfYB.data[1]
                rfz=rfYB.data[2]
                
                y=regOdbYB.nodes[0][counter].coordinates[1]
                z=regOdbYB.nodes[0][counter].coordinates[2]
                
                armY=y-y0
                armZ=z-z0
                
                momY1=rfy*armZ
                momY2=-rfz*armY
                momY=momY1+momY2
                sumMy=sumMy+momY
                
        res=res+ str(C)+'\t'+str(sumMy)+'\n'
        
        f.write(res)
        f.close()
   
def printStuff(modelName):
    odbName = modelName+'.odb'
    odb = session.openOdb(odbName)
    elements = odb.rootAssembly.elements
    
    session.openOdb(name=odbName)
    odbObj = session.odbs[odbName]
    session.pngOptions.setValues(imageSize=SIZE_ON_SCREEN)
    session.viewports['Viewport: 1'].setValues(displayedObject=odbObj)
        
        #Show deformation
    leaf = dgo.Leaf(leafType=DEFAULT_MODEL)
    session.viewports['Viewport: 1'].odbDisplay.displayGroup.replace(leaf=leaf)    
    session.viewports['Viewport: 1'].view.setValues(cameraPosition=(178.813, 0, 
            -0.0708677))
    session.viewports['Viewport: 1'].view.setValues(nearPlane=104.889, 
        farPlane=182.737, width=24.0845, height=8.89318, cameraPosition=(
        178.813, 0.112238, 0.0897802), cameraTarget=(35, 0.112238, 0.0897803))
    session.viewports['Viewport: 1'].view.setValues(cameraPosition=(178.813, 
        0.112238, 0.0897803), cameraUpVector=(0, 0.00163908, 0.999999))
    session.viewports['Viewport: 1'].odbDisplay.display.setValues(plotState=(
        SYMBOLS_ON_DEF, ))
    session.viewports['Viewport: 1'].odbDisplay.setSymbolVariable(
        variableLabel='RF', outputPosition=NODAL, refinement=(COMPONENT, 
        'RF2'), vectorQuantity=VECTOR_COMPONENT, )
    session.printToFile(fileName=modelName+'RF2', format=PNG, 
                        canvasObjects=(session.viewports['Viewport: 1'], ))
    
        #Show contact
    leaf = dgo.LeafFromElementSets(elementSets='SET-ELEMENTS-HALF')
    session.viewports['Viewport: 1'].odbDisplay.displayGroup.remove(leaf=leaf)
    session.viewports['Viewport: 1'].view.setValues(session.views['Iso'])
    session.viewports['Viewport: 1'].view.setValues(nearPlane=104.305, 
        farPlane=177.511, cameraPosition=(116.353, -85.1303, 73.1154), 
        cameraUpVector=(-0.57735, 0.518888, 0.630414), cameraTarget=(35, 
        3.70001, 3.8147e-006))
    session.viewports['Viewport: 1'].view.setValues(nearPlane=110.537, 
        farPlane=171.279, cameraPosition=(89.2846, -103.828, 73.1154), 
        cameraUpVector=(-0.415425, 0.655744, 0.630414), cameraTarget=(35, 
        3.70001, 3.8147e-006))
    session.viewports['Viewport: 1'].view.setValues(nearPlane=108.506, 
        farPlane=173.31, cameraPosition=(97.7684, -103.828, 65.9758), 
        cameraUpVector=(-0.335727, 0.655744, 0.67623), cameraTarget=(35, 
        3.70001, 4.76837e-006))
    session.viewports['Viewport: 1'].view.setValues(nearPlane=108.505, 
        farPlane=173.31, width=81.5414, height=30.1091, cameraPosition=(
        102.285, -102.104, 64.4894), cameraTarget=(39.5164, 5.4244, -1.4864))
    session.viewports['Viewport: 1'].odbDisplay.display.setValues(plotState=(
        CONTOURS_ON_DEF, ))
    session.viewports['Viewport: 1'].view.setValues(cameraPosition=(94.6188, 
        -105.295, 66.5816), cameraTarget=(31.8504, 2.23313, 0.605753))
    session.viewports['Viewport: 1'].odbDisplay.setPrimaryVariable(
        variableLabel='CPRESS', outputPosition=ELEMENT_NODAL, )
    session.printToFile(fileName=modelName+'CPRESS', format=PNG, 
                        canvasObjects=(session.viewports['Viewport: 1'], ))

# Main program
numSteps = int(sys.argv[-3])    
lineStart = int(sys.argv[-2])
numLines = int(sys.argv[-1])
for i in range(lineStart, lineStart+numLines):
        if i<10:
            ip = str(i)
            fileName = 'usefulFile-c-m00'+ip+'.txt'
        elif i<100:
            ip = str(i)
            fileName = 'usefulFile-c-m0'+ip+'.txt'
        else: 
            ip = str(i)
            fileName = 'usefulFile-c-m'+ip+'.txt'
        
        modelName, dX, dY, th, ky = getUsefulPar(fileName)
        results(modelName,dX,dY,th,ky)
        printStuff(modelName)
