# -*- coding: utf-8 -*-
"""
Created on Wed Apr 25 13:54:32 2018

@author: eivin
"""


from abaqus import *
from abaqusConstants import *
import __main__

import section
import regionToolset
import displayGroupMdbToolset as dgm
import part
import material
import assembly
import step
import interaction
import load
import mesh
import optimization
import job
import sketch
import visualization
import xyPlot
import displayGroupOdbToolset as dgo
import connectorBehavior
import sys
import os

def makeModel(modelName,dim1,dim2,dim3,dim4,thickness):
    
    theModel = mdb.Model(name='Model-1', modelType=STANDARD_EXPLICIT)
    
    # CREATING THE PART:
    
    s = theModel.ConstrainedSketch(name='__profile__', 
        sheetSize=100.0)
    g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
    s.setPrimaryObject(option=STANDALONE)
    # Note: no constraints or dimensions on the sketch needed, just the lines: 
    s.Line(point1=(0.0, 0.0), point2=(0.0, dim2+dim3))
    s.Line(point1=(0.0, dim2+dim3), point2=(dim4, dim2+dim3))
    s.Line(point1=(dim4, dim2+dim3), point2=(dim4, dim2+dim3+dim2))
    s.Line(point1=(dim4, dim2+dim3+dim2), point2=(dim4+dim1, dim2+dim3+dim2))
    s.Line(point1=(dim4+dim1, dim2+dim3+dim2), point2=(dim4+dim1, dim2))
    s.Line(point1=(dim4+dim1, dim2), point2=(dim1, dim2))
    s.Line(point1=(dim1, dim2), point2=(dim1, 0.0))
    s.Line(point1=(dim1, 0.0), point2=(0.0, 0.0))
    p = theModel.Part(name='BasePart', dimensionality=THREE_D, 
        type=DEFORMABLE_BODY)
    p.BaseSolidExtrude(sketch=s, depth=thickness)
    s.unsetPrimaryObject()
    del theModel.sketches['__profile__']

def copyBasePartToNewModel(modelName):
#    # Non-linear
#    mdb.Model(name=modelName, modelType=STANDARD_EXPLICIT)
#    mdb.models[modelName].Part('BasePart', mdb.models['Model-1'].parts['BasePart'])
    
    # Linear
    modelNameL = modelName + '-L'
    mdb.Model(name=modelNameL, modelType=STANDARD_EXPLICIT)
    mdb.models[modelNameL].Part('BasePartL', mdb.models['Model-1'].parts['BasePart'])


def meshAndMirror(modelName,esize,eType):
    
    # Linear Tensile
    modelNameL = modelName + '-L'
    m=mdb.models[modelNameL]
    p=m.parts['BasePartL']
    p.seedPart(size=esize, deviationFactor=0.1, minSizeFactor=0.1)
    print(eType) #For å sjå i ABAQUS om eg har rett namn på elementtypen
    
    
#    elemType2 = mesh.ElemType(elemCode=C3D20, elemLibrary=STANDARD)
    if eType == 'C3D20R':
        elemType1 = mesh.ElemType(elemCode=C3D20R, elemLibrary=STANDARD)
    elif eType ==  'C3D8R':
        elemType1 = mesh.ElemType(elemCode=C3D8R, elemLibrary=STANDARD)
        
    p.Set(cells=p.cells, name='Set-1')
    p.setElementType(regions=p.sets['Set-1'], elemTypes=(elemType1,))
    p.setMeshControls(regions=p.cells, technique=SWEEP)
    p.generateMesh()
    
    
    p.PartFromMesh(name='MeshPart1', copySets=True)    
    p1 = m.parts['MeshPart1']
    p2 = m.Part(name='MeshPart1-Copy-1', 
        objectToCopy=p1, compressFeatureList=ON, mirrorPlane=XYPLANE)
#    p3 = m.Part(name='MeshPart1-Copy-2', 
#        objectToCopy=p1, compressFeatureList=ON, mirrorPlane=XZPLANE)
#    p4 = m.Part(name='MeshPart1-Copy-3', 
#        objectToCopy=p1, compressFeatureList=ON, mirrorPlane=YZPLANE)
#    p5 = m.Part(name='MeshPart1-Copy-4', 
#        objectToCopy=p2, compressFeatureList=ON, mirrorPlane=YZPLANE)
#    p6 = m.Part(name='MeshPart1-Copy-5', 
#        objectToCopy=p2, compressFeatureList=ON, mirrorPlane=XZPLANE)
#    p7 = m.Part(name='MeshPart1-Copy-6', 
#        objectToCopy=p4, compressFeatureList=ON, mirrorPlane=XZPLANE)
#    p8 = m.Part(name='MeshPart1-Copy-7', 
#        objectToCopy=p5, compressFeatureList=ON, mirrorPlane=XZPLANE)
    
    a = m.rootAssembly
    a.DatumCsysByDefault(CARTESIAN)
    
    inst1=a.Instance(name='MeshPart1-1', part=p1, dependent=ON)
    inst2=a.Instance(name='MeshPart1-Copy-1', part=p2, dependent=ON)
#    inst3=a.Instance(name='MeshPart1-Copy-2', part=p3, dependent=ON)
#    inst4=a.Instance(name='MeshPart1-Copy-3', part=p4, dependent=ON)
#    inst5=a.Instance(name='MeshPart1-Copy-4', part=p5, dependent=ON)
#    inst6=a.Instance(name='MeshPart1-Copy-5', part=p6, dependent=ON)
#    inst7=a.Instance(name='MeshPart1-Copy-6', part=p7, dependent=ON)
#    inst8=a.Instance(name='MeshPart1-Copy-7', part=p8, dependent=ON)
    inst = a.InstanceFromBooleanMerge(name='MeshPart2', instances=(
           inst1, inst2, ), 
           mergeNodes=BOUNDARY_ONLY, nodeMergingTolerance=1e-06, domain=MESH, 
           originalInstances=DELETE)
    instName = 'MeshPart2-1'
    return(instName)

def materialAndProperties(modelName,Emod,Pois):
    
    # Linear Tenisle
    modelNameL = modelName + '-L'
    m=mdb.models[modelNameL]
    m.Material(name='Material-1')
    m.materials['Material-1'].Elastic(table=((Emod, Pois), ))
    m.HomogeneousSolidSection(name='Section-1', material='Material-1', thickness=None)
    p = m.parts['MeshPart2']
    reg = p.Set(elements=p.elements, name='SET-ELEMENTS-ALL')
    p.SectionAssignment(region=reg, sectionName='Section-1', offset=0.0, 
        offsetType=MIDDLE_SURFACE, offsetField='', 
        thicknessAssignment=FROM_SECTION)
    
def getInstanceDimensions(modelName,instanceName):
    xmax,ymax,zmax = 0.0, 0.0, 0.0
    a = mdb.models[modelName].rootAssembly  
    allNodes = a.instances[instanceName].nodes
    for n in allNodes:
        x,y,z = n.coordinates[0], n.coordinates[1], n.coordinates[2]
        xmax = max(xmax,x)
        ymax = max(ymax,y)
        zmax = max(zmax,z)
    return xmax,ymax,zmax

def copyBaseModelToNewModel(baseModelName,newModelName):
    mdb.Model(name=newModelName, objectToCopy=mdb.models[baseModelName])
    

def createAssemblySets(modelName,instanceName,dX,dY,dZ,tol):
    a = mdb.models[modelName].rootAssembly
    allNodes = a.instances[instanceName].nodes
    
    nodesXa = allNodes.getByBoundingBox(-dX-tol,-dY-tol,-dZ-tol,-dX+tol, dY+tol,dZ+tol)
    a.Set(nodes=nodesXa, name='SET-NODES-XA')
    
    nodesXb = allNodes.getByBoundingBox( dX-tol,-dY-tol,-dZ-tol, dX+tol, dY+tol,dZ+tol)
    a.Set(nodes=nodesXb, name='SET-NODES-XB')    
    
    nodesYa = allNodes.getByBoundingBox(-dX-tol,-dY-tol,-dZ-tol, dX+tol,-dY+tol,dZ+tol)
    a.Set(nodes=nodesYa, name='SET-NODES-YA')
    
    nodesYb = allNodes.getByBoundingBox(-dX-tol, dY-tol,-dZ-tol, dX+tol, dY+tol,dZ+tol)
    a.Set(nodes=nodesYb, name='SET-NODES-YB')
    
    nodeOrigo=allNodes.getByBoundingBox(0-tol, 0-tol, 0-tol, 0+tol, 0+tol, 0+tol)
    a.Set(nodes=nodeOrigo, name='SET-NODE-ORIGO')
    
    nodeZpos=allNodes.getByBoundingBox(0-tol, 0-tol, dZ-tol, 0+tol, 0+tol, dZ+tol)
    a.Set(nodes=nodeZpos, name='SET-NODE-ZPOS')
    
    nodeZneg=allNodes.getByBoundingBox(0-tol, 0-tol, -dZ-tol, 0+tol, 0+tol, -dZ+tol)
    a.Set(nodes=nodeZneg, name='SET-NODE-ZNEG')    
    

def stepAndBC(modelName,instanceName,dX,dY,dZ,tol,deformations):
    #Kxy=deformations[5]
    
    mod=mdb.models[modelName]
    mod.StaticStep(name='Step-1', previous=mod.steps.keys()[-1])
    a=mod.rootAssembly
        
    a.ReferencePoint(point=(-1.2*dX, 0.0, 0.0))
    refPoints=(a.referencePoints[a.features['RP-1'].id], )
    a.Set(referencePoints=refPoints, name='RPX')     
    
    a.ReferencePoint(point=(0.0, -1.2*dY, 0.0))
    refPoints=(a.referencePoints[a.features['RP-2'].id], )
    a.Set(referencePoints=refPoints, name='RPY') 
    
    a.ReferencePoint(point=(0.0, 0.0, -1.2*dZ))
    refPoints=(a.referencePoints[a.features['RP-3'].id], )
    a.Set(referencePoints=refPoints, name='RPZ') 

    mod.DisplacementBC(name='BCX', createStepName='Step-1', 
        region=a.sets['RPX'], u1=1.0, u2=1.0, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, 
        amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='', localCsys=None)
    mod.DisplacementBC(name='BCY', createStepName='Step-1', 
        region=a.sets['RPY'], u1=1.0, u2=1.0, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, 
        amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='', localCsys=None)
    mod.DisplacementBC(name='BCZ', createStepName='Step-1', 
        region=a.sets['RPZ'], u1=UNSET, u2=UNSET, u3=1.0, ur1=UNSET, ur2=UNSET, ur3=UNSET, 
        amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='', localCsys=None)    
    
    mod.DisplacementBC(name='BZZ', createStepName='Step-1', 
        region=a.sets['SET-NODE-ORIGO'], u1=UNSET, u2=UNSET, u3=0.0, ur1=UNSET, ur2=UNSET, ur3=UNSET, 
        amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='', localCsys=None)    

    mod.DisplacementBC(name='BRTOP', createStepName='Step-1', 
        region=a.sets['SET-NODE-ZPOS'], u1=0.0, u2=0.0, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, 
        amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='', localCsys=None)        
    mod.DisplacementBC(name='BRBOT', createStepName='Step-1', 
        region=a.sets['SET-NODE-ZNEG'], u1=0.0, u2=0.0, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, 
        amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, fieldName='', localCsys=None)        
    
    allNodes = a.instances[instanceName].nodes    
    nodesXa = allNodes.getByBoundingBox(-dX-tol,-dY-tol,-dZ-tol,-dX+tol, dY+tol,dZ+tol)    
    nodesXb = allNodes.getByBoundingBox( dX-tol,-dY-tol,-dZ-tol, dX+tol, dY+tol,dZ+tol)          
    nodesYa = allNodes.getByBoundingBox(-dX-tol,-dY-tol,-dZ-tol, dX-tol,-dY+tol,dZ+tol)
    nodesYb = allNodes.getByBoundingBox(-dX-tol, dY-tol,-dZ-tol, dX-tol, dY+tol,dZ+tol)    
    
    a.Set(nodes=nodesXa+nodesXb+nodesYa+nodesYb, name = 'Set-BoundaryNodes')
        
    mod.fieldOutputRequests['F-Output-1'].setValues(variables=(
        'S', 'PE', 'PEEQ', 'PEMAG', 'LE', 'U', 'RF', 'CF', 'NFORC', 'CSTRESS', 
        'CDISP', 'EVOL', 'COORD'))    
    
    
def constraintEquations(modelName,instanceName,dX,dY,dZ,tol,deformations):
    exx=deformations[0]
    eyy=deformations[1]
    exy=deformations[2]
    Kxx=deformations[3]
    Kyy=deformations[4]
    Kxy=deformations[5]
    
    mod = mdb.models[modelName]
    a = mod.rootAssembly
    allNodes = a.instances[instanceName].nodes
        
    nodesXa = allNodes.getByBoundingBox(-dX-tol,-dY-tol,-dZ-tol,-dX+tol, dY+tol,dZ+tol)    
    nodesXb = allNodes.getByBoundingBox( dX-tol,-dY-tol,-dZ-tol, dX+tol, dY+tol,dZ+tol)       
    
    counter=0
    for n in nodesXa:
        name1="Xa%i" % (counter)
        nodes1=nodesXa[counter:counter+1]
        a.Set(nodes=nodes1, name=name1)
        x,y,z = n.coordinates[0], n.coordinates[1], n.coordinates[2]
        name2="Xb%i" % (counter)
        nodes2=nodesXb.getByBoundingBox(x+2.0*dX-tol, y-tol, z-tol, x+2.0*dX+tol, y+tol, z+tol)
        a.Set(nodes=nodes2, name=name2)
        mod.Equation(name="Cq11x%i" % (counter), 
	    terms=((1.0, name2, 1), ( -1.0, name1, 1), ( -2.0*dX*(exx+z*Kxx), 'RPX', 1), )) 
        mod.Equation(name="Cq21x%i" % (counter), 
	    terms=((1.0, name2, 2), ( -1.0, name1, 2), ( -1.0*dX*(exy+z*Kxy), 'RPX', 2), ))              
        mod.Equation(name="Cqzx%i" % (counter), 
	    terms=((1.0, name2, 3), ( -1.0, name1, 3), ( +1.0*dX*y*Kxy, 'RPZ', 3), ))              
        
        counter=counter+1
    
    #Note: excluding the nodes at positive x:
    nodesYa = allNodes.getByBoundingBox(-dX-tol,-dY-tol,-dZ-tol, dX-tol,-dY+tol,dZ+tol)
    nodesYb = allNodes.getByBoundingBox(-dX-tol, dY-tol,-dZ-tol, dX-tol, dY+tol,dZ+tol)
    
    counter=0
    for n in nodesYa:
        name1="Ya%i" % (counter)
        nodes1=nodesYa[counter:counter+1]
        a.Set(nodes=nodes1, name=name1)
        x,y,z = n.coordinates[0], n.coordinates[1], n.coordinates[2]
        name2="Yb%i" % (counter)
        nodes2=nodesYb.getByBoundingBox(x-tol, y+2.0*dY-tol, z-tol, x+tol, y+2.0*dY+tol, z+tol)
        a.Set(nodes=nodes2, name=name2)
        mod.Equation(name="Cq12y%i" % (counter), 
	    terms=((1.0, name2, 1), ( -1.0, name1, 1), (-1.0*dY*(exy+z*Kxy), 'RPY', 1), ))   
        mod.Equation(name="Cq22y%i" % (counter), 
	    terms=((1.0, name2, 2), ( -1.0, name1, 2), (-2.0*dY*(eyy+z*Kyy), 'RPY', 2), ))            
        mod.Equation(name="Cqzy%i" % (counter), 
	    terms=((1.0, name2, 3), ( -1.0, name1, 3), (+1.0*dY*x*Kxy, 'RPZ', 3), ))
        counter=counter+1    
         

def createJob(modelName):
    jobName = modelName
    mdb.Job(name=jobName, model=modelName, description='', type=ANALYSIS, 
        atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90, 
        memoryUnits=PERCENTAGE, getMemoryFromAnalysis=True, 
        explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF, 
        modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine='', 
        scratch='', resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=1, 
        numGPUs=0)
    mdb.jobs[jobName].writeInput(consistencyChecking=OFF)

def runLoadCases(baseModelName,newModelName,instanceName,tol,loadCases):
    baseModelName = baseModelName + '-L'
    newModelName = baseModelName
    dX,dY,dZ=getInstanceDimensions(baseModelName,instanceName)
    for i in range(0,len(loadCases)):
        modelName=newModelName+'-'+str(i)
        copyBaseModelToNewModel(baseModelName,modelName)
        createAssemblySets(modelName,instanceName,dX,dY,dZ,tol)
        stepAndBC(modelName,instanceName,dX,dY,dZ,tol,loadCases[i])
        constraintEquations(modelName,instanceName,dX,dY,dZ,tol,loadCases[i])
        createJob(modelName)
    return(dX,dY,dZ)

    # Make dim for running
def makeDimensions(mdata):               #a,b,c,d
    dim1=float(mdata[6])                 #a
    dim2=float(mdata[9])                 #b
    dim3=float(mdata[8])                 #c
    dim4=float(mdata[6])+float(mdata[7]) #a+b
    th=float(mdata[10])
    return (dim1,dim2,dim3,dim4,th)

def makeUsefulFile(modelName,dX,dY,dZ,esize,tol):
    # Linear Tensile
    modelNameL = modelName + '-L'
    fileName = 'usefulFile-'+modelNameL+'.txt'
    f = open(fileName,'w')
    string = [modelNameL, '\t', str(dX), '\t', str(dY), '\t', str(dZ),'\t', str(esize),'\t', str(tol)]
    f.writelines(string)
    f.close()

# Main program
def runModel(modelName,esize,eType,Emod,Pois,dim1,dim2,dim3,dim4,thickness):
    modelName = 'ct-'+modelName
    makeModel(modelName,dim1,dim2,dim3,dim4,thickness)
    copyBasePartToNewModel(modelName)
    instance = meshAndMirror(modelName,esize,eType)
    materialAndProperties(modelName,Emod,Pois)
    
    dX,dY,dZ=runLoadCases(modelName,modelName,instance,esize*0.01,LC1)
    
    makeUsefulFile(modelName,dX,dY,dZ,esize,esize*0.01)


# =============================================================================
#   ### GLOBAL PARAMETERS ###
# =============================================================================

LC1=((1,0,0,0,0,0),(0,1,0,0,0,0),(0,0,1,0,0,0),(0,0,0,1,0,0),(0,0,0,0,1,0),(0,0,0,0,0,1))    
dataFileName=sys.argv[-1]
lineStart=int(sys.argv[-3])
lineStop=(lineStart-1) + int(sys.argv[-2])
#workDir = os.getcwd()
start = str(sys.argv[-3])
end = str(lineStop)
#dirName = 'Job_'+start+'_to_'+end   
#dataFile=open(workDir+'\\'+dataFileName)
dataFile=open(dataFileName)

modelsData=[]
for line in dataFile:
    dataList=[]
    data=line.split()
    for d in data:
        dataList.append(d.strip(' \t\n\r'))
    modelsData.append(dataList) 
    
dataFile.close()

if len(modelsData) < int(sys.argv[-2]):
    print('Too many jobs requested\n')
elif (int(sys.argv[-3])-1) < 0:
    print('Invalid start number!\n')    
elif (len(modelsData)-(lineStart-1)) < int(sys.argv[-2]):
    print('Not enough datalines left to give', int(sys.argv[-2]),'results\n')   
else:
    for i in range(lineStart-1, lineStop):
        mdata = modelsData[i]
        DIMS = makeDimensions(mdata)
        RadiusOfCurvature = float(mdata[10])
        MidPlaneStrain = float(mdata[9])
    
        #    modelName         esize            eName           emod               pois        dim1     dim2     dim3     dim4     th  
        runModel(mdata[0],  float(mdata[2]), mdata[3],  float(mdata[4]),  float(mdata[5]), DIMS[0], DIMS[1], DIMS[2], DIMS[3], DIMS[4])


